import { h, VirtualDOM } from '@lumino/virtualdom';
import { Drag } from 'tde-dragdrop';
import { MimeData } from '@lumino/coreutils';
// Class names for the shadow/line
const SHADOW = 'lm-DataGrid-select-shadow';
const LINE = 'lm-DataGrid-select-line';
function createRectangle(height, width) {
    return VirtualDOM.realize(h.div({
        className: SHADOW,
        style: {
            width: width.toString() + 'px',
            height: height.toString() + 'px'
        }
    }));
}
function createLine(height, width) {
    return VirtualDOM.realize(h.div({
        className: LINE,
        style: {
            width: `${width}px`,
            height: `${height}px`
        }
    }));
}
export function renderSelection(r1, r2, c1, c2, x, y, boundingRegion = null, type = 'line') {
    const height = r2 - r1;
    const width = c2 - c1;
    const mouseOffsetX = x - c1;
    const mouseOffsetY = y - r1;
    const target = type === 'line'
        ? createLine(height, width)
        : createRectangle(height, width);
    const dragSelection = new BoundedDrag({
        mimeData: new MimeData(),
        dragImage: target,
        proposedAction: 'move',
        boundingRegion,
        mouseOffsetX,
        mouseOffsetY
    });
    dragSelection.start(x, y).then(() => {
        return;
    });
    return dragSelection;
}
export class BoundedDrag extends Drag {
    constructor(options) {
        super(options);
        this._boundingRegion = options.boundingRegion;
        this._mouseOffsetX = options.mouseOffsetX;
        this._mouseOffsetY = options.mouseOffsetY;
        this._initializing = true;
        this.moveDragImage;
    }
    moveDragImage(clientX, clientY) {
        // see if we lack a drag image or if drag image is update-less
        if (!this.dragImage) {
            return;
        }
        if (this._boundingRegion || this._initializing) {
            let sudoClientX = clientX - this._mouseOffsetX;
            let sudoClientY = clientY - this._mouseOffsetY;
            [sudoClientX, sudoClientY] = this.bound(sudoClientX, sudoClientY);
            const style = this.dragImage.style;
            style.top = `${sudoClientY}px`;
            style.left = `${sudoClientX}px`;
            this._initializing = false;
        }
    }
    bound(x, y) {
        if (!this._boundingRegion) {
            return [x, y];
        }
        // unpack the bounding region
        const { topBound, bottomBound, leftBound, rightBound } = this._boundingRegion;
        // Fetch the style.
        const style = this.dragImage.style;
        // Fetch the image dimensions.
        const width = parseFloat(style.width);
        const height = parseFloat(style.height);
        // Bound
        x = Math.max(leftBound, Math.min(x, rightBound - width));
        y = Math.max(topBound, Math.min(y, bottomBound - height));
        return [x, y];
    }
    manualPositionUpdate(xLocation, yLocation) {
        // Bail early if there is already a bounding region
        if (this._boundingRegion) {
            return;
        }
        const style = this.dragImage.style;
        if (xLocation) {
            style.left = `${xLocation}px`;
        }
        if (yLocation) {
            style.top = `${yLocation}px`;
        }
    }
}
//# sourceMappingURL=drag.js.map